package sun.security.smartcardio;

import javax.smartcardio.CardNotPresentException;
import javax.smartcardio.CardException;
import javax.smartcardio.CardPermission;
import javax.smartcardio.Card;
import javax.smartcardio.CardTerminal;

final class TerminalImpl extends CardTerminal
{
    final long contextId;
    final String name;
    private CardImpl card;
    
    TerminalImpl(long contextId, String name) {
        this.contextId = contextId;
        this.name = name;
    }
    
    @Override
    public String getName() {
        return this.name;
    }
    
    @Override
    public synchronized Card connect(String str) throws CardException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new CardPermission(this.name, "connect"));
        }
        if (this.card != null) {
            if (this.card.isValid()) {
                String protocol = this.card.getProtocol();
                if (str.equals("*") || str.equalsIgnoreCase(protocol)) {
                    return this.card;
                }
                throw new CardException("Cannot connect using " + str + ", connection already established using " + protocol);
            }
            else {
                this.card = null;
            }
        }
        try {
            return this.card = new CardImpl(this, str);
        }
        catch (PCSCException ex) {
            if (ex.code == -2146434967 || ex.code == -2146435060) {
                throw new CardNotPresentException("No card present", ex);
            }
            throw new CardException("connect() failed", ex);
        }
    }
    
    @Override
    public boolean isCardPresent() throws CardException {
        try {
            return (PCSC.SCardGetStatusChange(this.contextId, 0L, new int[] { 0 }, new String[] { this.name })[0] & 0x20) != 0x0;
        }
        catch (PCSCException cause) {
            throw new CardException("isCardPresent() failed", cause);
        }
    }
    
    private boolean waitForCard(boolean b, long max) throws CardException {
        if (max < 0L) {
            throw new IllegalArgumentException("timeout must not be negative");
        }
        if (max == 0L) {
            max = -1L;
        }
        int[] array = { 0 };
        String[] array2 = { this.name };
        try {
            int[] array3 = PCSC.SCardGetStatusChange(this.contextId, 0L, array, array2);
            int n = (array3[0] & 0x20) != 0x0;
            if ((b ? 1 : 0) == n) {
                return true;
            }
            long n2 = System.currentTimeMillis() + max;
            while ((b ? 1 : 0) != n && max != 0L) {
                if (max != -1L) {
                    max = Math.max(n2 - System.currentTimeMillis(), 0L);
                }
                array3 = PCSC.SCardGetStatusChange(this.contextId, max, array3, array2);
                n = ((array3[0] & 0x20) != 0x0);
            }
            return (b ? 1 : 0) == n;
        }
        catch (PCSCException cause) {
            if (cause.code == -2146435062) {
                return false;
            }
            throw new CardException("waitForCard() failed", cause);
        }
    }
    
    @Override
    public boolean waitForCardPresent(long n) throws CardException {
        return this.waitForCard(true, n);
    }
    
    @Override
    public boolean waitForCardAbsent(long n) throws CardException {
        return this.waitForCard(false, n);
    }
    
    @Override
    public String toString() {
        return "PC/SC terminal " + this.name;
    }
}
